import torch
import torch.nn as nn
import torch.nn.functional as F
import pdb

from .EfficientDet.utils_eff import EFFICIENTDET, get_state_dict

class CrowdCounter(nn.Module):
    def __init__(self,gpus,model_name):
        super(CrowdCounter, self).__init__()        

        print(model_name)

        num_class = 10
        network = 'efficientnet-b3'

        if model_name == 'EfficientDet':
            from .EfficientDet.efficientdet import EfficientDet as net
        elif model_name == 'EfficientFPN':
            from .efficientNet.model import EfficientFPN as net

        if model_name == 'EfficientDet':
           self.CCN = net(num_classes=num_class,
                         network=network,
                         W_bifpn=EFFICIENTDET[network]['W_bifpn'],
                         D_bifpn=EFFICIENTDET[network]['D_bifpn'],
                         D_class=EFFICIENTDET[network]['D_class']
                      )
        elif model_name == 'EfficientFPN':
            self.CCN = net(network)

        #self.CCN = net()
        if len(gpus)>1:
            self.CCN = torch.nn.DataParallel(self.CCN, device_ids=gpus).cuda()
        else:
            self.CCN=self.CCN.cuda()
        self.loss_mse_fn = nn.MSELoss().cuda()
        
    @property
    def loss(self):
        return self.loss_mse
    
    def forward(self, img, gt_map):                               
        density_map = self.CCN(img)                          
        self.loss_mse= self.build_loss(density_map.squeeze(), gt_map.squeeze())               
        return density_map
    
    def build_loss(self, density_map, gt_data):
        #import IPython; IPython.embed()
        loss_mse = self.loss_mse_fn(density_map, gt_data)  
        return loss_mse

    def test_forward(self, img):                               
        density_map = self.CCN(img)                    
        return density_map

