import torch
import torch.nn as nn
import torch.nn.functional as F
import pdb

class CrowdCounter(nn.Module):
    def __init__(self,gpus,model_name):
        super(CrowdCounter, self).__init__()        

        if model_name == 'Res50_RHC':
            from .RHCCC_Model.Res50 import Res50 as net
        elif model_name == 'Res101_RHC':
            from .RHCCC_Model.Res101 import Res101 as net
        elif model_name == 'EfficientNet_RHC':
            from .RHCCC_Model.efficientNet.model import EfficientNet as net
        elif model_name == 'CSRNet_RHC':
            from .RHCCC_Model.CSRNet import CSRNet as net
        elif model_name == 'SFCN_RHC':
            from .RHCCC_Model.Res101_SFCN import Res101_SFCN as net

        self.CCN = net()
        if len(gpus)>1:
            self.CCN = torch.nn.DataParallel(self.CCN, device_ids=gpus).cuda()
        else:
            self.CCN=self.CCN.cuda()
        self.loss_mse_fn = nn.MSELoss().cuda()
        
    @property
    def loss(self):
        return self.loss_mse, self.loss_mse_p2, 
    
    def forward(self, img, gt_map, gt_map_min):                               
        density_map, density_map_min = self.CCN(img)                          
        #import IPython; IPython.embed()

        self.loss_mse = self.build_loss(
            density_map.squeeze(), 
            gt_map.squeeze())

        self.loss_mse_p2 = self.build_loss(
            density_map_min.squeeze(), 
            gt_map_min.squeeze())

                      
        return density_map
    
    def build_loss(self, density_map, gt_data):
        loss_mse = self.loss_mse_fn(density_map, gt_data) 
        return loss_mse

    def test_forward(self, img):                               
        density_map, _ = self.CCN(img)                    
        return density_map

