
from matplotlib import pyplot as plt

import matplotlib
import os
import random
import torch
from torch.autograd import Variable
import torchvision.transforms as standard_transforms
import misc.transforms as own_transforms
import pandas as pd

from models.RHCC import CrowdCounter
from config import cfg
from misc.utils import *
import scipy.io as sio
from PIL import Image, ImageOps

torch.cuda.set_device(0)
torch.backends.cudnn.benchmark = True

exp_name = '../RHC_results/CSRNet_RHC_test'
if not os.path.exists(exp_name):
    os.mkdir(exp_name)

if not os.path.exists(exp_name+'/pred'):
    os.mkdir(exp_name+'/pred')

if not os.path.exists(exp_name+'/gt'):
    os.mkdir(exp_name+'/gt')

#mean_std = ([0.452016860247, 0.447249650955, 0.431981861591],[0.23242045939, 0.224925786257, 0.221840232611])
mean_std = ([0.410824894905, 0.370634973049, 0.359682112932], [0.278580576181, 0.26925137639, 0.27156367898])
img_transform = standard_transforms.Compose([
        standard_transforms.ToTensor(),
        standard_transforms.Normalize(*mean_std)
    ])
restore = standard_transforms.Compose([
        own_transforms.DeNormalize(*mean_std),
        standard_transforms.ToPILImage()
    ])
pil_to_tensor = standard_transforms.ToTensor()

dataRoot = '../ProcessedData/RHC/test'

model_path = '../02-20_09-10_RHC_CSRNet_RHC_0.0001/all_ep_166_mae_0.2_mse_0.3.pth'
#model_path = '../C-3-Framework/exp/02-23_08-38_RHC_Res101_RHC_0.0001/all_ep_94_mae_0.2_mse_0.4.pth'
#model_path = '../02-21_06-54_RHC_SFCN_RHC_0.0001/all_ep_116_mae_0.2_mse_0.5.pth'
#model_path = '../02-22_13-58_RHC2_Res50_0.0001/all_ep_141_mae_0.2_mse_0.4.pth'

def main():
    
    file_list = [filename for root,dirs,filename in os.walk(dataRoot+'/img/')]

    test(file_list[0], model_path)
   

def test(file_list, model_path):

    net = CrowdCounter(cfg.GPU_ID,cfg.NET)
    net.load_state_dict(torch.load(model_path))
    net.cuda()
    net.eval()

    #f1 = plt.figure(1)

    gts = []
    preds = []
    sqeerr = []

    for filename in file_list:
        print(filename)
        #import IPython; IPython.embed()
        imgname = dataRoot + '/img/' + filename
        filename_no_ext = filename.replace('.jpg','')

        denname = dataRoot + '/den/' + filename_no_ext + '_gt' + '.csv'

        den = pd.read_csv(denname, sep=',',header=None).values
        den = den.astype(np.float32, copy=False)

        img = Image.open(imgname)

        if img.mode == 'L':
            img = img.convert('RGB')

        img = img_transform(img)

        gt = np.sum(den)
        with torch.no_grad():
            img = Variable(img[None,:,:,:]).cuda()
            pred_map = net.test_forward(img)

        #sio.savemat(exp_name+'/pred/'+filename_no_ext+'.mat',{'data':pred_map.squeeze().cpu().numpy()/100.})
        #sio.savemat(exp_name+'/gt/'+filename_no_ext+'.mat',{'data':den})

        pred_map = pred_map.cpu().data.numpy()[0,0,:,:]

        pred = np.sum(pred_map)/100000.0
        pred_map = pred_map/np.max(pred_map+1e-20)
        
        den = den/np.max(den+1e-20)

        #import IPython; IPython.embed()
        print(gt)
        print(pred)
        diff = pred - gt
        sqr = np.power(diff,2)
        sqeerr.append([filename_no_ext,gt,pred,sqr])

        SaveDensityMap(pred_map,filename_no_ext,pred)

        #den_frame = plt.gca()
        #plt.imshow(den, 'jet')
        #den_frame.axes.get_yaxis().set_visible(False)
        #den_frame.axes.get_xaxis().set_visible(False)
        #den_frame.spines['top'].set_visible(False) 
        #den_frame.spines['bottom'].set_visible(False) 
        #den_frame.spines['left'].set_visible(False) 
        #den_frame.spines['right'].set_visible(False) 
        #plt.savefig(exp_name+'/'+filename_no_ext+'_gt_'+str(int(gt))+'.png',\
        #    bbox_inches='tight',pad_inches=0,dpi=150)

        #plt.close()
        
        # sio.savemat(exp_name+'/'+filename_no_ext+'_gt_'+str(int(gt))+'.mat',{'data':den})

        #pred_frame = plt.gca()
        #plt.imshow(pred_map, 'jet')
        #pred_frame.axes.get_yaxis().set_visible(False)
        #pred_frame.axes.get_xaxis().set_visible(False)
        #pred_frame.spines['top'].set_visible(False) 
        #pred_frame.spines['bottom'].set_visible(False) 
        #pred_frame.spines['left'].set_visible(False) 
        #pred_frame.spines['right'].set_visible(False) 
        #plt.savefig(exp_name+'/'+filename_no_ext+'_pred_'+str(float(pred))+'.png',\
        #    bbox_inches='tight',pad_inches=0,dpi=150)

        #plt.close()

        # sio.savemat(exp_name+'/'+filename_no_ext+'_pred_'+str(float(pred))+'.mat',{'data':pred_map})

        #diff = den-pred_map

        #diff_frame = plt.gca()
        #plt.imshow(diff, 'jet')
        #plt.colorbar()
        #diff_frame.axes.get_yaxis().set_visible(False)
        #diff_frame.axes.get_xaxis().set_visible(False)
        #diff_frame.spines['top'].set_visible(False) 
        #diff_frame.spines['bottom'].set_visible(False) 
        #diff_frame.spines['left'].set_visible(False) 
        #diff_frame.spines['right'].set_visible(False) 
        #plt.savefig(exp_name+'/'+filename_no_ext+'_diff.png',\
        #    bbox_inches='tight',pad_inches=0,dpi=150)

        #plt.close()

        # sio.savemat(exp_name+'/'+filename_no_ext+'_diff.mat',{'data':diff})
    
    np.savetxt(exp_name+'/Hasil.csv',sqeerr,delimiter=",", fmt="%s")


def SaveDensityMap(density_map,filename,pred):
    
    height = 360
    width = 640

    density_map_fig = np.zeros((height,width,3))
    density_map_norm = (density_map-density_map.min())/(density_map.max()-density_map.min())

    for i in range(width):
        for j in range(height):
        #=========================================================================
        # Blue
        #=========================================================================
            if(density_map_norm[j,i]<=0.25):
                density_map_fig[j,i,2]=255
            #=========================================================================
            elif(density_map_norm[j,i]>0.25 and density_map_norm[j,i]<=0.5):
                density_map_fig[j,i,2]=255*(1-((density_map_norm[j,i]-0.25)/0.25))
            #=========================================================================
            elif(density_map_norm[j,i]>0.5):
                density_map_fig[j,i,2]=0
            #=========================================================================
                
        #=========================================================================
        # Green
        #=========================================================================
            if(density_map_norm[j,i]<=0.25):
                density_map_fig[j,i,1]=255*((density_map_norm[j,i])/0.25)
            #=========================================================================
            elif(density_map_norm[j,i]>0.25 and density_map_norm[j,i]<=0.75):
                density_map_fig[j,i,1]=255
            #=========================================================================
            elif(density_map_norm[j,i]>0.75):
                density_map_fig[j,i,1]=255*(1-((density_map_norm[j,i]-0.75)/0.25))
            #=========================================================================
                
        #=========================================================================
        # Red
        #=========================================================================
            if(density_map_norm[j,i]<=0.5):
                density_map_fig[j,i,0]=0
            #=========================================================================
            elif(density_map_norm[j,i]>0.5 and density_map_norm[j,i]<=0.75):
                density_map_fig[j,i,0]=255*((density_map_norm[j,i]-0.5)/0.25)
            #=========================================================================
            elif(density_map_norm[j,i]>0.75):
                density_map_fig[j,i,0]=255
            #=========================================================================
    
    pred_img = Image.fromarray(density_map_fig.astype(np.uint8))
    pred_img.save(exp_name+'/'+filename+'_pred_'+str(int(pred))+'.jpg')


if __name__ == '__main__':
    main()




